@@ -27,13 +27,17 @@
 #include <curl/curl.h>
 
 #include "urldata.h"
+#include "url.h"
 #include "cfilters.h"
 #include "curl_trc.h"
 #include "multiif.h"
-#include "timeval.h"
+#include "curlx/timeval.h"
 #include "multi_ev.h"
 #include "select.h"
-#include "warnless.h"
+#include "uint-bset.h"
+#include "uint-spbset.h"
+#include "uint-table.h"
+#include "curlx/warnless.h"
 #include "multihandle.h"
 #include "socks.h"
 /* The last 3 #include files should be in this order */
@@ -47,33 +51,34 @@ static void mev_in_callback(struct Curl_multi *multi, bool value)
   multi->in_callback = value;
 }
 
-#define CURL_MEV_XFER_HASH_SIZE 13
 #define CURL_MEV_CONN_HASH_SIZE 3
 
 /* Information about a socket for which we inform the libcurl application
  * what to supervise (CURL_POLL_IN/CURL_POLL_OUT/CURL_POLL_REMOVE)
  */
 struct mev_sh_entry {
-  struct Curl_hash_offt xfers; /* hash of transfers using this socket */
-  struct Curl_hash_offt conns; /* hash of connections using this socket */
+  struct uint_spbset xfers; /* bitset of transfers `mid`s on this socket */
+  struct connectdata *conn; /* connection using this socket or NULL */
   void *user_data;      /* libcurl app data via curl_multi_assign() */
   unsigned int action;  /* CURL_POLL_IN/CURL_POLL_OUT we last told the
                          * libcurl application to watch out for */
   unsigned int readers; /* this many transfers want to read */
   unsigned int writers; /* this many transfers want to write */
+  BIT(announced);       /* this socket has been passed to the socket
+                           callback at least once */
 };
 
 static size_t mev_sh_entry_hash(void *key, size_t key_length, size_t slots_num)
 {
   curl_socket_t fd = *((curl_socket_t *) key);
-  (void) key_length;
+  (void)key_length;
   return (fd % (curl_socket_t)slots_num);
 }
 
 static size_t mev_sh_entry_compare(void *k1, size_t k1_len,
                                    void *k2, size_t k2_len)
 {
-  (void) k1_len; (void) k2_len;
+  (void)k1_len; (void)k2_len;
   return (*((curl_socket_t *) k1)) == (*((curl_socket_t *) k2));
 }
 
@@ -81,8 +86,7 @@ static size_t mev_sh_entry_compare(void *k1, size_t k1_len,
 static void mev_sh_entry_dtor(void *freethis)
 {
   struct mev_sh_entry *entry = (struct mev_sh_entry *)freethis;
-  Curl_hash_offt_destroy(&entry->xfers);
-  Curl_hash_offt_destroy(&entry->conns);
+  Curl_uint_spbset_destroy(&entry->xfers);
   free(entry);
 }
 
@@ -114,8 +118,7 @@ mev_sh_entry_add(struct Curl_hash *sh, curl_socket_t s)
   if(!check)
     return NULL; /* major failure */
 
-  Curl_hash_offt_init(&check->xfers, CURL_MEV_XFER_HASH_SIZE, NULL);
-  Curl_hash_offt_init(&check->conns, CURL_MEV_CONN_HASH_SIZE, NULL);
+  Curl_uint_spbset_init(&check->xfers);
 
   /* make/add new hash entry */
   if(!Curl_hash_add(sh, (char *)&s, sizeof(curl_socket_t), check)) {
@@ -134,19 +137,19 @@ static void mev_sh_entry_kill(struct Curl_multi *multi, curl_socket_t s)
 
 static size_t mev_sh_entry_user_count(struct mev_sh_entry *e)
 {
-  return Curl_hash_offt_count(&e->xfers) + Curl_hash_offt_count(&e->conns);
+  return Curl_uint_spbset_count(&e->xfers) + (e->conn ? 1 : 0);
 }
 
 static bool mev_sh_entry_xfer_known(struct mev_sh_entry *e,
                                     struct Curl_easy *data)
 {
-  return !!Curl_hash_offt_get(&e->xfers, data->mid);
+  return Curl_uint_spbset_contains(&e->xfers, data->mid);
 }
 
 static bool mev_sh_entry_conn_known(struct mev_sh_entry *e,
                                     struct connectdata *conn)
 {
-  return !!Curl_hash_offt_get(&e->conns, conn->connection_id);
+  return (e->conn == conn);
 }
 
 static bool mev_sh_entry_xfer_add(struct mev_sh_entry *e,
@@ -154,7 +157,7 @@ static bool mev_sh_entry_xfer_add(struct mev_sh_entry *e,
 {
    /* detect weird values */
   DEBUGASSERT(mev_sh_entry_user_count(e) < 100000);
-  return !!Curl_hash_offt_set(&e->xfers, data->mid, data);
+  return Curl_uint_spbset_add(&e->xfers, data->mid);
 }
 
 static bool mev_sh_entry_conn_add(struct mev_sh_entry *e,
@@ -162,20 +165,32 @@ static bool mev_sh_entry_conn_add(struct mev_sh_entry *e,
 {
    /* detect weird values */
   DEBUGASSERT(mev_sh_entry_user_count(e) < 100000);
-  return !!Curl_hash_offt_set(&e->conns, conn->connection_id, conn);
+  DEBUGASSERT(!e->conn);
+  if(e->conn)
+    return FALSE;
+  e->conn = conn;
+  return TRUE;
 }
 
 
 static bool mev_sh_entry_xfer_remove(struct mev_sh_entry *e,
                                      struct Curl_easy *data)
 {
-  return Curl_hash_offt_remove(&e->xfers, data->mid);
+  bool present = Curl_uint_spbset_contains(&e->xfers, data->mid);
+  if(present)
+    Curl_uint_spbset_remove(&e->xfers, data->mid);
+  return present;
 }
 
 static bool mev_sh_entry_conn_remove(struct mev_sh_entry *e,
                                      struct connectdata *conn)
 {
-  return Curl_hash_offt_remove(&e->conns, conn->connection_id);
+  DEBUGASSERT(e->conn == conn);
+  if(e->conn == conn) {
+    e->conn = NULL;
+    return TRUE;
+  }
+  return FALSE;
 }
 
 /* Purge any information about socket `s`.
@@ -192,13 +207,14 @@ static CURLMcode mev_forget_socket(struct Curl_multi *multi,
     return CURLM_OK;
 
   /* We managed this socket before, tell the socket callback to forget it. */
-  if(multi->socket_cb) {
+  if(entry->announced && multi->socket_cb) {
     CURL_TRC_M(data, "ev %s, call(fd=%" FMT_SOCKET_T ", ev=REMOVE)",
                cause, s);
     mev_in_callback(multi, TRUE);
     rc = multi->socket_cb(data, s, CURL_POLL_REMOVE,
                           multi->socket_userp, entry->user_data);
     mev_in_callback(multi, FALSE);
+    entry->announced = FALSE;
   }
 
   mev_sh_entry_kill(multi, s);
@@ -268,8 +284,8 @@ static CURLMcode mev_sh_entry_update(struct Curl_multi *multi,
   mev_in_callback(multi, TRUE);
   rc = multi->socket_cb(data, s, comboaction, multi->socket_userp,
                         entry->user_data);
-
   mev_in_callback(multi, FALSE);
+  entry->announced = TRUE;
   if(rc == -1) {
     multi->dead = TRUE;
     return CURLM_ABORTED_BY_CALLBACK;
@@ -290,7 +306,7 @@ static CURLMcode mev_pollset_diff(struct Curl_multi *multi,
   CURLMcode mresult;
 
   /* The transfer `data` reports in `ps` the sockets it is interested
-   * in and which combinatino of CURL_POLL_IN/CURL_POLL_OUT it wants
+   * in and which combination of CURL_POLL_IN/CURL_POLL_OUT it wants
    * to have monitored for events.
    * There can be more than 1 transfer interested in the same socket
    * and 1 transfer might be interested in more than 1 socket.
@@ -301,7 +317,7 @@ static CURLMcode mev_pollset_diff(struct Curl_multi *multi,
   DEBUGASSERT(prev_ps);
 
   /* Handle changes to sockets the transfer is interested in. */
-  for(i = 0; i < ps->num; i++) {
+  for(i = 0; i < ps->n; i++) {
     unsigned char last_action;
     bool first_time = FALSE; /* data/conn appears first time on socket */
 
@@ -339,14 +355,14 @@ static CURLMcode mev_pollset_diff(struct Curl_multi *multi,
           return CURLM_OUT_OF_MEMORY;
       }
       CURL_TRC_M(data, "ev entry fd=%" FMT_SOCKET_T ", added %s #%" FMT_OFF_T
-                 ", total=%zu/%zu (xfer/conn)", s,
+                 ", total=%u/%d (xfer/conn)", s,
                  conn ? "connection" : "transfer",
                  conn ? conn->connection_id : data->mid,
-                 Curl_hash_offt_count(&entry->xfers),
-                 Curl_hash_offt_count(&entry->conns));
+                 Curl_uint_spbset_count(&entry->xfers),
+                 entry->conn ? 1 : 0);
     }
     else {
-      for(j = 0; j < prev_ps->num; j++) {
+      for(j = 0; j < prev_ps->n; j++) {
         if(s == prev_ps->sockets[j]) {
           last_action = prev_ps->actions[j];
           break;
@@ -361,11 +377,11 @@ static CURLMcode mev_pollset_diff(struct Curl_multi *multi,
   }
 
   /* Handle changes to sockets the transfer is NO LONGER interested in. */
-  for(i = 0; i < prev_ps->num; i++) {
+  for(i = 0; i < prev_ps->n; i++) {
     bool stillused = FALSE;
 
     s = prev_ps->sockets[i];
-    for(j = 0; j < ps->num; j++) {
+    for(j = 0; j < ps->n; j++) {
       if(s == ps->sockets[j]) {
         /* socket is still supervised */
         stillused = TRUE;
@@ -409,9 +425,9 @@ static CURLMcode mev_pollset_diff(struct Curl_multi *multi,
       if(mresult)
         return mresult;
       CURL_TRC_M(data, "ev entry fd=%" FMT_SOCKET_T ", removed transfer, "
-                 "total=%zu/%zu (xfer/conn)", s,
-                 Curl_hash_offt_count(&entry->xfers),
-                 Curl_hash_offt_count(&entry->conns));
+                 "total=%u/%d (xfer/conn)", s,
+                 Curl_uint_spbset_count(&entry->xfers),
+                 entry->conn ? 1 : 0);
     }
     else {
       mresult = mev_forget_socket(multi, data, s, "last user gone");
@@ -421,76 +437,100 @@ static CURLMcode mev_pollset_diff(struct Curl_multi *multi,
   } /* for loop over num */
 
   /* Remember for next time */
-  memcpy(prev_ps, ps, sizeof(*prev_ps));
+  Curl_pollset_move(prev_ps, ps);
   return CURLM_OK;
 }
 
+static void mev_pollset_dtor(void *key, size_t klen, void *entry)
+{
+  struct easy_pollset *ps = entry;
+  (void)key;
+  (void)klen;
+  if(ps) {
+    Curl_pollset_cleanup(ps);
+    free(ps);
+  }
+}
+
 static struct easy_pollset*
-mev_add_new_pollset(struct Curl_hash_offt *h, curl_off_t id)
+mev_add_new_conn_pollset(struct connectdata *conn)
 {
   struct easy_pollset *ps;
 
-  ps = calloc(1, sizeof(*ps));
+  ps = Curl_pollset_create();
   if(!ps)
     return NULL;
-  if(!Curl_hash_offt_set(h, id, ps)) {
-    free(ps);
+  if(Curl_conn_meta_set(conn, CURL_META_MEV_POLLSET, ps, mev_pollset_dtor))
+    return NULL;
+  return ps;
+}
+
+static struct easy_pollset*
+mev_add_new_xfer_pollset(struct Curl_easy *data)
+{
+  struct easy_pollset *ps;
+
+  ps = Curl_pollset_create();
+  if(!ps)
+    return NULL;
+  if(Curl_meta_set(data, CURL_META_MEV_POLLSET, ps, mev_pollset_dtor))
     return NULL;
-  }
   return ps;
 }
 
 static struct easy_pollset *
-mev_get_last_pollset(struct Curl_multi *multi,
-                     struct Curl_easy *data,
+mev_get_last_pollset(struct Curl_easy *data,
                      struct connectdata *conn)
 {
   if(data) {
     if(conn)
-      return Curl_hash_offt_get(&multi->ev.conn_pollsets,
-                                conn->connection_id);
-    return Curl_hash_offt_get(&multi->ev.xfer_pollsets, data->mid);
+      return Curl_conn_meta_get(conn, CURL_META_MEV_POLLSET);
+    return Curl_meta_get(data, CURL_META_MEV_POLLSET);
   }
   return NULL;
 }
 
-static void mev_init_cur_pollset(struct easy_pollset *ps,
-                                 struct Curl_easy *data,
-                                 struct connectdata *conn)
-{
-  memset(ps, 0, sizeof(*ps));
-  if(conn)
-    Curl_conn_adjust_pollset(data, conn, ps);
-  else if(data)
-    Curl_multi_getsock(data, ps, "ev assess");
-}
-
 static CURLMcode mev_assess(struct Curl_multi *multi,
                             struct Curl_easy *data,
                             struct connectdata *conn)
 {
-  if(multi && multi->socket_cb) {
-    struct easy_pollset ps, *last_ps;
+  struct easy_pollset ps, *last_ps;
+  CURLMcode res = CURLM_OK;
 
-    mev_init_cur_pollset(&ps, data, conn);
-    last_ps = mev_get_last_pollset(multi, data, conn);
+  if(!multi || !multi->socket_cb)
+    return CURLM_OK;
 
-    if(!last_ps && ps.num) {
-      if(conn)
-        last_ps = mev_add_new_pollset(&multi->ev.conn_pollsets,
-                                      conn->connection_id);
-      else
-        last_ps = mev_add_new_pollset(&multi->ev.xfer_pollsets, data->mid);
-      if(!last_ps)
-        return CURLM_OUT_OF_MEMORY;
+  Curl_pollset_init(&ps);
+  if(conn) {
+    CURLcode r = Curl_conn_adjust_pollset(data, conn, &ps);
+    if(r) {
+      res = (r == CURLE_OUT_OF_MEMORY) ?
+            CURLM_OUT_OF_MEMORY : CURLM_INTERNAL_ERROR;
+      goto out;
     }
-
-    if(last_ps)
-      return mev_pollset_diff(multi, data, conn, &ps, last_ps);
-    else
-      DEBUGASSERT(!ps.num);
   }
-  return CURLM_OK;
+  else if(data)
+    Curl_multi_pollset(data, &ps, "ev assess");
+  last_ps = mev_get_last_pollset(data, conn);
+
+  if(!last_ps && ps.n) {
+    if(conn)
+      last_ps = mev_add_new_conn_pollset(conn);
+    else
+      last_ps = mev_add_new_xfer_pollset(data);
+    if(!last_ps) {
+      res = CURLM_OUT_OF_MEMORY;
+      goto out;
+    }
+  }
+
+  if(last_ps)
+    res = mev_pollset_diff(multi, data, conn, &ps, last_ps);
+  else
+    DEBUGASSERT(!ps.n);
+out:
+  Curl_pollset_cleanup(&ps);
+  return res;
 }
 
 CURLMcode Curl_multi_ev_assess_xfer(struct Curl_multi *multi,
@@ -506,16 +546,19 @@ CURLMcode Curl_multi_ev_assess_conn(struct Curl_multi *multi,
   return mev_assess(multi, data, conn);
 }
 
-CURLMcode Curl_multi_ev_assess_xfer_list(struct Curl_multi *multi,
-                                         struct Curl_llist *list)
+CURLMcode Curl_multi_ev_assess_xfer_bset(struct Curl_multi *multi,
+                                         struct uint_bset *set)
 {
-  struct Curl_llist_node *e;
+  unsigned int mid;
   CURLMcode result = CURLM_OK;
 
-  if(multi && multi->socket_cb) {
-    for(e = Curl_llist_head(list); e && !result; e = Curl_node_next(e)) {
-      result = Curl_multi_ev_assess_xfer(multi, Curl_node_elem(e));
+  if(multi && multi->socket_cb && Curl_uint_bset_first(set, &mid)) {
+    do {
+      struct Curl_easy *data = Curl_multi_get_easy(multi, mid);
+      if(data)
+        result = Curl_multi_ev_assess_xfer(multi, data);
     }
+    while(!result && Curl_uint_bset_next(set, mid, &mid));
   }
   return result;
 }
@@ -532,25 +575,9 @@ CURLMcode Curl_multi_ev_assign(struct Curl_multi *multi,
   return CURLM_OK;
 }
 
-static bool mev_xfer_expire_cb(curl_off_t id, void *value, void *user_data)
-{
-  const struct curltime *nowp = user_data;
-  struct Curl_easy *data = value;
-
-  DEBUGASSERT(data);
-  DEBUGASSERT(data->magic == CURLEASY_MAGIC_NUMBER);
-  if(data && id >= 0) {
-    /* Expire with out current now, so we will get it below when
-     * asking the splaytree for expired transfers. */
-    Curl_expire_ex(data, nowp, 0, EXPIRE_RUN_NOW);
-  }
-  return TRUE;
-}
-
-void Curl_multi_ev_expire_xfers(struct Curl_multi *multi,
-                                curl_socket_t s,
-                                const struct curltime *nowp,
-                                bool *run_cpool)
+void Curl_multi_ev_dirty_xfers(struct Curl_multi *multi,
+                               curl_socket_t s,
+                               bool *run_cpool)
 {
   struct mev_sh_entry *entry;
 
@@ -563,10 +590,24 @@ void Curl_multi_ev_expire_xfers(struct Curl_multi *multi,
      asked to get removed, so thus we better survive stray socket actions
      and just move on. */
   if(entry) {
-    Curl_hash_offt_visit(&entry->xfers, mev_xfer_expire_cb,
-                         CURL_UNCONST(nowp));
+    struct Curl_easy *data;
+    unsigned int mid;
 
-    if(Curl_hash_offt_count(&entry->conns))
+    if(Curl_uint_spbset_first(&entry->xfers, &mid)) {
+      do {
+        data = Curl_multi_get_easy(multi, mid);
+        if(data) {
+          Curl_multi_mark_dirty(data);
+        }
+        else {
+          CURL_TRC_M(multi->admin, "socket transfer %u no longer found", mid);
+          Curl_uint_spbset_remove(&entry->xfers, mid);
+        }
+      }
+      while(Curl_uint_spbset_next(&entry->xfers, mid, &mid));
+    }
+
+    if(entry->conn)
       *run_cpool = TRUE;
   }
 }
@@ -581,9 +622,9 @@ void Curl_multi_ev_xfer_done(struct Curl_multi *multi,
                              struct Curl_easy *data)
 {
   DEBUGASSERT(!data->conn); /* transfer should have been detached */
-  if(data->mid >= 0) {
+  if(data != multi->admin) {
     (void)mev_assess(multi, data, NULL);
-    Curl_hash_offt_remove(&multi->ev.xfer_pollsets, data->mid);
+    Curl_meta_remove(data, CURL_META_MEV_POLLSET);
   }
 }
 
@@ -592,30 +633,18 @@ void Curl_multi_ev_conn_done(struct Curl_multi *multi,
                              struct connectdata *conn)
 {
   (void)mev_assess(multi, data, conn);
-  Curl_hash_offt_remove(&multi->ev.conn_pollsets, conn->connection_id);
+  Curl_conn_meta_remove(conn, CURL_META_MEV_POLLSET);
 }
 
 #define CURL_MEV_PS_HASH_SLOTS   (991)  /* nice prime */
 
-static void mev_hash_pollset_free(curl_off_t id, void *entry)
-{
-  (void)id;
-  free(entry);
-}
-
 void Curl_multi_ev_init(struct Curl_multi *multi, size_t hashsize)
 {
   Curl_hash_init(&multi->ev.sh_entries, hashsize, mev_sh_entry_hash,
                  mev_sh_entry_compare, mev_sh_entry_dtor);
-  Curl_hash_offt_init(&multi->ev.xfer_pollsets,
-                      CURL_MEV_PS_HASH_SLOTS, mev_hash_pollset_free);
-  Curl_hash_offt_init(&multi->ev.conn_pollsets,
-                      CURL_MEV_PS_HASH_SLOTS, mev_hash_pollset_free);
 }
 
 void Curl_multi_ev_cleanup(struct Curl_multi *multi)
 {
   Curl_hash_destroy(&multi->ev.sh_entries);
-  Curl_hash_offt_destroy(&multi->ev.xfer_pollsets);
-  Curl_hash_offt_destroy(&multi->ev.conn_pollsets);
 }
