@@ -23,11 +23,9 @@
  ***************************************************************************/
 #include "tool_setup.h"
 
-#include "strcase.h"
-
-#define ENABLE_CURLX_PRINTF
-/* use our own printf() functions */
-#include "curlx.h"
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 #include "tool_cfgable.h"
 #include "tool_doswin.h"
@@ -36,6 +34,7 @@
 #include "tool_cb_wrt.h"
 #include "tool_operate.h"
 #include "tool_libinfo.h"
+#include "tool_strdup.h"
 
 #include "memdebug.h" /* keep this as LAST include */
 
@@ -47,7 +46,7 @@ static char *parse_filename(const char *ptr, size_t len);
 #else
 #define BOLD "\x1b[1m"
 /* Switch off bold by setting "all attributes off" since the explicit
-   bold-off code (21) isn't supported everywhere - like in the mac
+   bold-off code (21) is not supported everywhere - like in the mac
    Terminal. */
 #define BOLDOFF "\x1b[0m"
 /* OSC 8 hyperlink escape sequence */
@@ -58,13 +57,31 @@ static char *parse_filename(const char *ptr, size_t len);
 
 #ifdef LINK
 static void write_linked_location(CURL *curl, const char *location,
-    size_t loclen, FILE *stream);
+                                  size_t loclen, FILE *stream);
 #endif
 
+int tool_write_headers(struct HdrCbData *hdrcbdata, FILE *stream)
+{
+  struct curl_slist *h = hdrcbdata->headlist;
+  int rc = 1;
+  while(h) {
+    /* not "handled", just show it */
+    size_t len = strlen(h->data);
+    if(len != fwrite(h->data, 1, len, stream))
+      goto fail;
+    h = h->next;
+  }
+  rc = 0; /* success */
+fail:
+  curl_slist_free_all(hdrcbdata->headlist);
+  hdrcbdata->headlist = NULL;
+  return rc;
+}
+
+
 /*
 ** callback for CURLOPT_HEADERFUNCTION
 */
-
 size_t tool_header_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
 {
   struct per_transfer *per = userdata;
@@ -82,7 +99,7 @@ size_t tool_header_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
 
 #ifdef DEBUGBUILD
   if(size * nmemb > (size_t)CURL_MAX_HTTP_HEADER) {
-    warnf(per->config->global, "Header data exceeds single call write limit");
+    warnf("Header data exceeds write limit");
     return CURL_WRITEFUNC_ERROR;
   }
 #endif
@@ -102,7 +119,10 @@ size_t tool_header_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
     if(rc != cb)
       return rc;
     /* flush the stream to send off what we got earlier */
-    (void)fflush(heads->stream);
+    if(fflush(heads->stream)) {
+      errorf("Failed writing headers to %s", per->config->headerfile);
+      return CURL_WRITEFUNC_ERROR;
+    }
   }
 
   curl_easy_getinfo(per->curl, CURLINFO_SCHEME, &scheme);
@@ -111,8 +131,9 @@ size_t tool_header_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
     long response = 0;
     curl_easy_getinfo(per->curl, CURLINFO_RESPONSE_CODE, &response);
 
-    if(response/100 != 2)
-      /* only care about these headers in 2xx responses */
+    if((response/100 != 2) && (response/100 != 3))
+      /* only care about etag and content-disposition headers in 2xx and 3xx
+         responses */
       ;
     /*
      * Write etag to file when --etag-save option is given.
@@ -130,6 +151,19 @@ size_t tool_header_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
 
         if(eot >= etag_h) {
           size_t etag_length = eot - etag_h + 1;
+          /*
+           * Truncate the etag save stream, it can have an existing etag value.
+           */
+#if defined(HAVE_FTRUNCATE) && !defined(__MINGW32CE__)
+          if(ftruncate(fileno(etag_save->stream), 0)) {
+            return CURL_WRITEFUNC_ERROR;
+          }
+#else
+          if(fseek(etag_save->stream, 0, SEEK_SET)) {
+            return CURL_WRITEFUNC_ERROR;
+          }
+#endif
+
           fwrite(etag_h, size, etag_length, etag_save->stream);
           /* terminate with newline */
           fputc('\n', etag_save->stream);
@@ -145,66 +179,90 @@ size_t tool_header_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
      * Content-Disposition header specifying a filename property.
      */
 
-    else if(hdrcbdata->honor_cd_filename &&
-            (cb > 20) && checkprefix("Content-disposition:", str)) {
-      const char *p = str + 20;
+    else if(hdrcbdata->honor_cd_filename) {
+      if((cb > 20) && checkprefix("Content-disposition:", str)) {
+        const char *p = str + 20;
 
-      /* look for the 'filename=' parameter
-         (encoded filenames (*=) are not supported) */
-      for(;;) {
-        char *filename;
-        size_t len;
+        /* look for the 'filename=' parameter
+           (encoded filenames (*=) are not supported) */
+        for(;;) {
+          char *filename;
+          size_t len;
 
-        while((p < end) && *p && !ISALPHA(*p))
-          p++;
-        if(p > end - 9)
-          break;
-
-        if(memcmp(p, "filename=", 9)) {
-          /* no match, find next parameter */
-          while((p < end) && *p && (*p != ';'))
+          while((p < end) && *p && !ISALPHA(*p))
             p++;
-          if((p < end) && *p)
-            continue;
-          else
+          if(p > end - 9)
             break;
-        }
-        p += 9;
 
-        /* this expression below typecasts 'cb' only to avoid
-           warning: signed and unsigned type in conditional expression
-        */
-        len = (ssize_t)cb - (p - str);
-        filename = parse_filename(p, len);
-        if(filename) {
-          if(outs->stream) {
-            /* indication of problem, get out! */
-            free(filename);
-            return CURL_WRITEFUNC_ERROR;
+          if(memcmp(p, "filename=", 9)) {
+            /* no match, find next parameter */
+            while((p < end) && *p && (*p != ';'))
+              p++;
+            if((p < end) && *p)
+              continue;
+            else
+              break;
           }
+          p += 9;
 
-          if(per->config->output_dir) {
-            outs->filename = aprintf("%s/%s", per->config->output_dir,
-                                     filename);
-            free(filename);
-            if(!outs->filename)
+          len = cb - (size_t)(p - str);
+          filename = parse_filename(p, len);
+          if(filename) {
+            if(outs->stream) {
+              /* indication of problem, get out! */
+              free(filename);
+              return CURL_WRITEFUNC_ERROR;
+            }
+
+            if(per->config->output_dir) {
+              outs->filename = aprintf("%s/%s", per->config->output_dir,
+                                       filename);
+              free(filename);
+              if(!outs->filename)
+                return CURL_WRITEFUNC_ERROR;
+            }
+            else
+              outs->filename = filename;
+
+            outs->is_cd_filename = TRUE;
+            outs->s_isreg = TRUE;
+            outs->fopened = FALSE;
+            outs->alloc_filename = TRUE;
+            hdrcbdata->honor_cd_filename = FALSE; /* done now! */
+            if(!tool_create_output_file(outs, per->config))
+              return CURL_WRITEFUNC_ERROR;
+            if(tool_write_headers(&per->hdrcbdata, outs->stream))
               return CURL_WRITEFUNC_ERROR;
           }
-          else
-            outs->filename = filename;
-
-          outs->is_cd_filename = TRUE;
-          outs->s_isreg = TRUE;
-          outs->fopened = FALSE;
-          outs->alloc_filename = TRUE;
-          hdrcbdata->honor_cd_filename = FALSE; /* done now! */
-          if(!tool_create_output_file(outs, per->config))
-            return CURL_WRITEFUNC_ERROR;
+          break;
         }
-        break;
+        if(!outs->stream && !tool_create_output_file(outs, per->config))
+          return CURL_WRITEFUNC_ERROR;
+        if(tool_write_headers(&per->hdrcbdata, outs->stream))
+          return CURL_WRITEFUNC_ERROR;
+      } /* content-disposition handling */
+
+      if(hdrcbdata->honor_cd_filename &&
+         hdrcbdata->config->show_headers) {
+        /* still awaiting the Content-Disposition header, store the header in
+           memory. Since it is not null-terminated, we need an extra dance. */
+        char *clone = aprintf("%.*s", (int)cb, str);
+        if(clone) {
+          struct curl_slist *old = hdrcbdata->headlist;
+          hdrcbdata->headlist = curl_slist_append(old, clone);
+          free(clone);
+          if(!hdrcbdata->headlist) {
+            curl_slist_free_all(old);
+            return CURL_WRITEFUNC_ERROR;
+          }
+        }
+        else {
+          curl_slist_free_all(hdrcbdata->headlist);
+          hdrcbdata->headlist = NULL;
+          return CURL_WRITEFUNC_ERROR;
+        }
+        return cb; /* done for now */
       }
-      if(!outs->stream && !tool_create_output_file(outs, per->config))
-        return CURL_WRITEFUNC_ERROR;
     }
   }
   if(hdrcbdata->config->writeout) {
@@ -227,11 +285,11 @@ size_t tool_header_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
     if(!outs->stream && !tool_create_output_file(outs, per->config))
       return CURL_WRITEFUNC_ERROR;
 
-    if(hdrcbdata->global->isatty &&
+    if(global->isatty &&
 #ifdef _WIN32
        tool_term_has_bold &&
 #endif
-       hdrcbdata->global->styled_output)
+       global->styled_output)
       value = memchr(ptr, ':', cb);
     if(value) {
       size_t namelen = value - ptr;
@@ -255,7 +313,7 @@ size_t tool_header_cb(char *ptr, size_t size, size_t nmemb, void *userdata)
 }
 
 /*
- * Copies a file name part and returns an ALLOCATED data buffer.
+ * Copies a filename part and returns an ALLOCATED data buffer.
  */
 static char *parse_filename(const char *ptr, size_t len)
 {
@@ -264,12 +322,9 @@ static char *parse_filename(const char *ptr, size_t len)
   char *q;
   char  stop = '\0';
 
-  /* simple implementation of strndup() */
-  copy = malloc(len + 1);
+  copy = memdup0(ptr, len);
   if(!copy)
     return NULL;
-  memcpy(copy, ptr, len);
-  copy[len] = '\0';
 
   p = copy;
   if(*p == '\'' || *p == '"') {
@@ -290,24 +345,24 @@ static char *parse_filename(const char *ptr, size_t len)
   if(q) {
     p = q + 1;
     if(!*p) {
-      Curl_safefree(copy);
+      tool_safefree(copy);
       return NULL;
     }
   }
 
   /* If the filename contains a backslash, only use filename portion. The idea
-     is that even systems that don't handle backslashes as path separators
+     is that even systems that do not handle backslashes as path separators
      probably want the path removed for convenience. */
   q = strrchr(p, '\\');
   if(q) {
     p = q + 1;
     if(!*p) {
-      Curl_safefree(copy);
+      tool_safefree(copy);
       return NULL;
     }
   }
 
-  /* make sure the file name doesn't end in \r or \n */
+  /* make sure the filename does not end in \r or \n */
   q = strchr(p, '\r');
   if(q)
     *q = '\0';
@@ -323,33 +378,13 @@ static char *parse_filename(const char *ptr, size_t len)
   {
     char *sanitized;
     SANITIZEcode sc = sanitize_file_name(&sanitized, copy, 0);
-    Curl_safefree(copy);
+    tool_safefree(copy);
     if(sc)
       return NULL;
     copy = sanitized;
   }
 #endif /* _WIN32 || MSDOS */
 
-  /* in case we built debug enabled, we allow an environment variable
-   * named CURL_TESTDIR to prefix the given file name to put it into a
-   * specific directory
-   */
-#ifdef DEBUGBUILD
-  {
-    char *tdir = curlx_getenv("CURL_TESTDIR");
-    if(tdir) {
-      char buffer[512]; /* suitably large */
-      msnprintf(buffer, sizeof(buffer), "%s/%s", tdir, copy);
-      Curl_safefree(copy);
-      copy = strdup(buffer); /* clone the buffer, we don't use the libcurl
-                                aprintf() or similar since we want to use the
-                                same memory code as the "real" parse_filename
-                                function */
-      curl_free(tdir);
-    }
-  }
-#endif
-
   return copy;
 }
 
@@ -358,27 +393,28 @@ static char *parse_filename(const char *ptr, size_t len)
  * Treat the Location: header specially, by writing a special escape
  * sequence that adds a hyperlink to the displayed text. This makes
  * the absolute URL of the redirect clickable in supported terminals,
- * which couldn't happen otherwise for relative URLs. The Location:
+ * which could not happen otherwise for relative URLs. The Location:
  * header is supposed to always be absolute so this theoretically
- * shouldn't be needed but the real world returns plenty of relative
+ * should not be needed but the real world returns plenty of relative
  * URLs here.
  */
-static
-void write_linked_location(CURL *curl, const char *location, size_t loclen,
-                           FILE *stream) {
+static void write_linked_location(CURL *curl, const char *location,
+                                  size_t loclen, FILE *stream)
+{
   /* This would so simple if CURLINFO_REDIRECT_URL were available here */
   CURLU *u = NULL;
   char *copyloc = NULL, *locurl = NULL, *scheme = NULL, *finalurl = NULL;
   const char *loc = location;
   size_t llen = loclen;
   int space_skipped = 0;
-  char *vver = getenv("VTE_VERSION");
+  const char *vver = getenv("VTE_VERSION");
 
   if(vver) {
-    long vvn = strtol(vver, NULL, 10);
-    /* Skip formatting for old versions of VTE <= 0.48.1 (Mar 2017) since some
-       of those versions have formatting bugs. (#10428) */
-    if(0 < vvn && vvn <= 4801)
+    curl_off_t num;
+    if(curlx_str_number(&vver, &num, CURL_OFF_T_MAX) ||
+       /* Skip formatting for old versions of VTE <= 0.48.1 (Mar 2017) since
+          some of those versions have formatting bugs. (#10428) */
+       (num <= 4801))
       goto locout;
   }
 
@@ -398,12 +434,10 @@ void write_linked_location(CURL *curl, const char *location, size_t loclen,
   if(!u)
     goto locout;
 
-  /* Create a NUL-terminated and whitespace-stripped copy of Location: */
-  copyloc = malloc(llen + 1);
+  /* Create a null-terminated and whitespace-stripped copy of Location: */
+  copyloc = memdup0(loc, llen);
   if(!copyloc)
     goto locout;
-  memcpy(copyloc, loc, llen);
-  copyloc[llen] = 0;
 
   /* The original URL to use as a base for a relative redirect URL */
   if(curl_easy_getinfo(curl, CURLINFO_EFFECTIVE_URL, &locurl))
@@ -432,7 +466,7 @@ void write_linked_location(CURL *curl, const char *location, size_t loclen,
     goto locdone;
   }
 
-  /* Not a "safe" URL: don't linkify it */
+  /* Not a "safe" URL: do not linkify it */
 
 locout:
   /* Write the normal output in case of error or unsafe */
