@@ -472,7 +472,8 @@ static SANITIZEcode rename_if_reserved_dos(char **const sanitized,
 
   /* Rename reserved device names that are known to be accessible without \\.\
      Examples: CON => _CON, CON.EXT => CON_EXT, CON:ADS => CON_ADS
-     https://support.microsoft.com/en-us/kb/74496
+     https://web.archive.org/web/20160314141551/
+       support.microsoft.com/en-us/kb/74496
      https://msdn.microsoft.com/en-us/library/windows/desktop/aa365247.aspx
      */
   for(p = fname; p; p = (p == fname && fname != base ? base : NULL)) {
@@ -746,13 +747,11 @@ CURLcode win32_init(void)
    by nmap and ncat (https://nmap.org/ncat/) */
 struct win_thread_data {
   /* This is a copy of the true stdin file handle before any redirection. It is
-      read by the thread. */
+     read by the thread. */
   HANDLE stdin_handle;
   /* This is the listen socket for the thread. It is closed after the first
-      connection. */
+     connection. */
   curl_socket_t socket_l;
-  /* This is the global config - used for printing errors and so forth */
-  struct GlobalConfig *global;
 };
 
 static DWORD WINAPI win_stdin_thread_func(void *thread_data)
@@ -766,18 +765,18 @@ static DWORD WINAPI win_stdin_thread_func(void *thread_data)
   SOCKADDR_IN clientAddr;
   int clientAddrLen = sizeof(clientAddr);
 
-  curl_socket_t socket_w = accept(tdata->socket_l, (SOCKADDR*)&clientAddr,
-                           &clientAddrLen);
+  curl_socket_t socket_w = CURL_ACCEPT(tdata->socket_l, (SOCKADDR*)&clientAddr,
+                                       &clientAddrLen);
 
   if(socket_w == CURL_SOCKET_BAD) {
-    errorf(tdata->global, "accept error: %08lx\n", GetLastError());
+    errorf("accept error: %08lx", GetLastError());
     goto ThreadCleanup;
   }
 
   closesocket(tdata->socket_l); /* sclose here fails test 1498 */
   tdata->socket_l = CURL_SOCKET_BAD;
   if(shutdown(socket_w, SD_RECEIVE) == SOCKET_ERROR) {
-    errorf(tdata->global, "shutdown error: %08lx\n", GetLastError());
+    errorf("shutdown error: %08lx", GetLastError());
     goto ThreadCleanup;
   }
   for(;;) {
@@ -813,7 +812,7 @@ ThreadCleanup:
 static HANDLE stdin_thread = NULL;
 static curl_socket_t socket_r = CURL_SOCKET_BAD;
 
-curl_socket_t win32_stdin_read_thread(struct GlobalConfig *global)
+curl_socket_t win32_stdin_read_thread(void)
 {
   int result;
   bool r;
@@ -831,7 +830,7 @@ curl_socket_t win32_stdin_read_thread(struct GlobalConfig *global)
     /* Prepare handles for thread */
     tdata = (struct win_thread_data*)calloc(1, sizeof(struct win_thread_data));
     if(!tdata) {
-      errorf(global, "calloc() error");
+      errorf("calloc() error");
       break;
     }
     /* Create the listening socket for the thread. When it starts, it will
@@ -840,7 +839,7 @@ curl_socket_t win32_stdin_read_thread(struct GlobalConfig *global)
                                  IPPROTO_TCP, NULL, 0, WSA_FLAG_OVERLAPPED);
 
     if(tdata->socket_l == CURL_SOCKET_BAD) {
-      errorf(global, "WSASocketW error: %08lx", GetLastError());
+      errorf("WSASocketW error: %08lx", GetLastError());
       break;
     }
 
@@ -851,20 +850,20 @@ curl_socket_t win32_stdin_read_thread(struct GlobalConfig *global)
     /* Bind to any available loopback port */
     result = bind(tdata->socket_l, (SOCKADDR*)&selfaddr, socksize);
     if(result == SOCKET_ERROR) {
-      errorf(global, "bind error: %08lx", GetLastError());
+      errorf("bind error: %08lx", GetLastError());
       break;
     }
 
     /* Bind to any available loopback port */
     result = getsockname(tdata->socket_l, (SOCKADDR*)&selfaddr, &socksize);
     if(result == SOCKET_ERROR) {
-      errorf(global, "getsockname error: %08lx", GetLastError());
+      errorf("getsockname error: %08lx", GetLastError());
       break;
     }
 
     result = listen(tdata->socket_l, 1);
     if(result == SOCKET_ERROR) {
-      errorf(global, "listen error: %08lx\n", GetLastError());
+      errorf("listen error: %08lx", GetLastError());
       break;
     }
 
@@ -874,7 +873,7 @@ curl_socket_t win32_stdin_read_thread(struct GlobalConfig *global)
                         0, FALSE, DUPLICATE_SAME_ACCESS);
 
     if(!r) {
-      errorf(global, "DuplicateHandle error: %08lx", GetLastError());
+      errorf("DuplicateHandle error: %08lx", GetLastError());
       break;
     }
 
@@ -885,14 +884,14 @@ curl_socket_t win32_stdin_read_thread(struct GlobalConfig *global)
     stdin_thread = CreateThread(NULL, 0, win_stdin_thread_func,
                                 tdata, 0, NULL);
     if(!stdin_thread) {
-      errorf(global, "CreateThread error: %08lx", GetLastError());
+      errorf("CreateThread error: %08lx", GetLastError());
       break;
     }
 
     /* Connect to the thread and rearrange our own STDIN handles */
     socket_r = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
     if(socket_r == CURL_SOCKET_BAD) {
-      errorf(global, "socket error: %08lx", GetLastError());
+      errorf("socket error: %08lx", GetLastError());
       break;
     }
 
@@ -900,18 +899,18 @@ curl_socket_t win32_stdin_read_thread(struct GlobalConfig *global)
     setsockopt(socket_r, SOL_SOCKET, SO_DONTLINGER, 0, 0);
 
     if(connect(socket_r, (SOCKADDR*)&selfaddr, socksize) == SOCKET_ERROR) {
-      errorf(global, "connect error: %08lx", GetLastError());
+      errorf("connect error: %08lx", GetLastError());
       break;
     }
 
     if(shutdown(socket_r, SD_SEND) == SOCKET_ERROR) {
-      errorf(global, "shutdown error: %08lx", GetLastError());
+      errorf("shutdown error: %08lx", GetLastError());
       break;
     }
 
     /* Set the stdin handle to read from the socket. */
     if(SetStdHandle(STD_INPUT_HANDLE, (HANDLE)socket_r) == 0) {
-      errorf(global, "SetStdHandle error: %08lx", GetLastError());
+      errorf("SetStdHandle error: %08lx", GetLastError());
       break;
     }
 
@@ -921,11 +920,11 @@ curl_socket_t win32_stdin_read_thread(struct GlobalConfig *global)
   if(rc != 1) {
     if(socket_r != CURL_SOCKET_BAD && tdata) {
       if(GetStdHandle(STD_INPUT_HANDLE) == (HANDLE)socket_r &&
-        tdata->stdin_handle) {
-          /* restore STDIN */
-          SetStdHandle(STD_INPUT_HANDLE, tdata->stdin_handle);
-          tdata->stdin_handle = NULL;
-        }
+         tdata->stdin_handle) {
+        /* restore STDIN */
+        SetStdHandle(STD_INPUT_HANDLE, tdata->stdin_handle);
+        tdata->stdin_handle = NULL;
+      }
 
       sclose(socket_r);
       socket_r = CURL_SOCKET_BAD;
