@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2021, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,33 +18,28 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
 /*
  * This source code is used for lib1502, lib1503, lib1504 and lib1505 with
- * only #ifdefs controlling the cleanup sequence.
+ * only the testnum controlling the cleanup sequence.
  *
  * Test case 1502 converted from bug report #3575448, identifying a memory
  * leak in the CURLOPT_RESOLVE handling with the multi interface.
  */
 
-#include "test.h"
+#include "first.h"
 
-#include <limits.h>
-
-#include "testutil.h"
-#include "warnless.h"
 #include "memdebug.h"
 
-#define TEST_HANG_TIMEOUT 60 * 1000
-
-int test(char *URL)
+static CURLcode test_lib1502(const char *URL)
 {
   CURL *easy = NULL;
   CURL *dup;
   CURLM *multi = NULL;
   int still_running;
-  int res = 0;
-
+  CURLcode res = CURLE_OK;
   char redirect[160];
 
   /* DNS cache injection */
@@ -55,14 +50,14 @@ int test(char *URL)
     return res;
   }
 
-  msnprintf(redirect, sizeof(redirect), "google.com:%s:%s", libtest_arg2,
-            libtest_arg3);
+  curl_msnprintf(redirect, sizeof(redirect), "google.com:%s:%s", libtest_arg2,
+                 libtest_arg3);
 
   start_test_timing();
 
   dns_cache_list = curl_slist_append(NULL, redirect);
   if(!dns_cache_list) {
-    fprintf(stderr, "curl_slist_append() failed\n");
+    curl_mfprintf(stderr, "curl_slist_append() failed\n");
     curl_global_cleanup();
     return TEST_ERR_MAJOR_BAD;
   }
@@ -121,35 +116,35 @@ int test(char *URL)
 
 test_cleanup:
 
-#ifdef LIB1502
-  /* undocumented cleanup sequence - type UA */
-  curl_multi_cleanup(multi);
-  curl_easy_cleanup(easy);
-  curl_global_cleanup();
-#endif
-
-#ifdef LIB1503
-  /* proper cleanup sequence - type PA */
-  curl_multi_remove_handle(multi, easy);
-  curl_multi_cleanup(multi);
-  curl_easy_cleanup(easy);
-  curl_global_cleanup();
-#endif
-
-#ifdef LIB1504
-  /* undocumented cleanup sequence - type UB */
-  curl_easy_cleanup(easy);
-  curl_multi_cleanup(multi);
-  curl_global_cleanup();
-#endif
-
-#ifdef LIB1505
-  /* proper cleanup sequence - type PB */
-  curl_multi_remove_handle(multi, easy);
-  curl_easy_cleanup(easy);
-  curl_multi_cleanup(multi);
-  curl_global_cleanup();
-#endif
+  switch(testnum) {
+  case 1502:
+  default:
+    /* undocumented cleanup sequence - type UA */
+    curl_multi_cleanup(multi);
+    curl_easy_cleanup(easy);
+    curl_global_cleanup();
+    break;
+  case 1503:
+    /* proper cleanup sequence - type PA */
+    curl_multi_remove_handle(multi, easy);
+    curl_multi_cleanup(multi);
+    curl_easy_cleanup(easy);
+    curl_global_cleanup();
+    break;
+  case 1504:
+    /* undocumented cleanup sequence - type UB */
+    curl_easy_cleanup(easy);
+    curl_multi_cleanup(multi);
+    curl_global_cleanup();
+    break;
+  case 1505:
+    /* proper cleanup sequence - type PB */
+    curl_multi_remove_handle(multi, easy);
+    curl_easy_cleanup(easy);
+    curl_multi_cleanup(multi);
+    curl_global_cleanup();
+    break;
+  }
 
   curl_slist_free_all(dns_cache_list);
 
