@@ -28,7 +28,7 @@
  * of 42 (CURLE_ABORTED_BY_CALLBACK).
  */
 
-#include "test.h"
+#include "first.h"
 
 #include "memdebug.h"
 
@@ -47,7 +47,7 @@ static int progressKiller(void *arg,
   return 1;
 }
 
-CURLcode test(char *URL)
+static CURLcode test_lib1513(const char *URL)
 {
   CURL *curl;
   CURLcode res = CURLE_OK;
@@ -57,11 +57,11 @@ CURLcode test(char *URL)
   easy_init(curl);
 
   easy_setopt(curl, CURLOPT_URL, URL);
-  easy_setopt(curl, CURLOPT_TIMEOUT, (long)7);
-  easy_setopt(curl, CURLOPT_NOSIGNAL, (long)1);
+  easy_setopt(curl, CURLOPT_TIMEOUT, 7L);
+  easy_setopt(curl, CURLOPT_NOSIGNAL, 1L);
   easy_setopt(curl, CURLOPT_PROGRESSFUNCTION, progressKiller);
   easy_setopt(curl, CURLOPT_PROGRESSDATA, NULL);
-  easy_setopt(curl, CURLOPT_NOPROGRESS, (long)0);
+  easy_setopt(curl, CURLOPT_NOPROGRESS, 0L);
 
   res = curl_easy_perform(curl);
 
