@@ -21,12 +21,10 @@
  * SPDX-License-Identifier: curl
  *
  ***************************************************************************/
-#include "test.h"
+#include "first.h"
 
 /* test case and code based on https://github.com/curl/curl/issues/3927 */
 
-#include "testutil.h"
-#include "warnless.h"
 #include "memdebug.h"
 
 static int dload_progress_cb(void *a, curl_off_t b, curl_off_t c,
@@ -40,7 +38,7 @@ static int dload_progress_cb(void *a, curl_off_t b, curl_off_t c,
   return 0;
 }
 
-static size_t write_cb(char *d, size_t n, size_t l, void *p)
+static size_t t1523_write_cb(char *d, size_t n, size_t l, void *p)
 {
   /* take care of the data here, ignored in this example */
   (void)d;
@@ -55,7 +53,7 @@ static CURLcode run(CURL *hnd, long limit, long time)
   return curl_easy_perform(hnd);
 }
 
-CURLcode test(char *URL)
+static CURLcode test_lib1523(const char *URL)
 {
   CURLcode ret;
   CURL *hnd;
@@ -63,18 +61,18 @@ CURLcode test(char *URL)
   curl_global_init(CURL_GLOBAL_ALL);
   hnd = curl_easy_init();
   curl_easy_setopt(hnd, CURLOPT_URL, URL);
-  curl_easy_setopt(hnd, CURLOPT_WRITEFUNCTION, write_cb);
+  curl_easy_setopt(hnd, CURLOPT_WRITEFUNCTION, t1523_write_cb);
   curl_easy_setopt(hnd, CURLOPT_ERRORBUFFER, buffer);
   curl_easy_setopt(hnd, CURLOPT_NOPROGRESS, 0L);
   curl_easy_setopt(hnd, CURLOPT_XFERINFOFUNCTION, dload_progress_cb);
 
   ret = run(hnd, 1, 2);
   if(ret)
-    fprintf(stderr, "error (%d) %s\n", ret, buffer);
+    curl_mfprintf(stderr, "error (%d) %s\n", ret, buffer);
 
   ret = run(hnd, 12000, 1);
   if(ret != CURLE_OPERATION_TIMEDOUT)
-    fprintf(stderr, "error (%d) %s\n", ret, buffer);
+    curl_mfprintf(stderr, "error (%d) %s\n", ret, buffer);
   else
     ret = CURLE_OK;
 
