@@ -21,16 +21,16 @@
  * SPDX-License-Identifier: curl
  *
  ***************************************************************************/
-#include "test.h"
+#include "first.h"
 
 #include "memdebug.h"
 
 #include <curl/multi.h>
 
-int test(char *URL)
+static CURLcode test_lib1550(const char *URL)
 {
   CURLM *handle;
-  int res = CURLE_OK;
+  CURLcode res = CURLE_OK;
   static const char * const bl_servers[] =
      {"Microsoft-IIS/6.0", "nginx/0.8.54", NULL};
   static const char * const bl_sites[] =
@@ -38,11 +38,11 @@ int test(char *URL)
 
   global_init(CURL_GLOBAL_ALL);
   handle = curl_multi_init();
-  (void)URL; /* unused */
+  (void)URL;
 
   curl_multi_setopt(handle, CURLMOPT_PIPELINING_SERVER_BL, bl_servers);
   curl_multi_setopt(handle, CURLMOPT_PIPELINING_SITE_BL, bl_sites);
   curl_multi_cleanup(handle);
   curl_global_cleanup();
-  return 0;
+  return CURLE_OK;
 }
