@@ -421,7 +421,7 @@ static int t518_test_rlimit(int keep_open)
   return 0;
 }
 
-static CURLcode test_lib518(char *URL)
+static CURLcode test_lib518(const char *URL)
 {
   CURLcode res;
   CURL *curl;
@@ -471,13 +471,13 @@ test_cleanup:
   return res;
 }
 
-#else /* defined(HAVE_GETRLIMIT) && defined(HAVE_SETRLIMIT) */
+#else /* HAVE_GETRLIMIT && HAVE_SETRLIMIT */
 
-static CURLcode test_lib518(char *URL)
+static CURLcode test_lib518(const char *URL)
 {
   (void)URL;
   curl_mprintf("system lacks necessary system function(s)");
   return TEST_ERR_MAJOR_BAD; /* skip test */
 }
 
-#endif /* defined(HAVE_GETRLIMIT) && defined(HAVE_SETRLIMIT) */
+#endif /* HAVE_GETRLIMIT && HAVE_SETRLIMIT */
