@@ -30,55 +30,47 @@
  * argv3 = proxyuser:password
  */
 
-#include "test.h"
-#include "testutil.h"
-#include "warnless.h"
+#include "first.h"
+
 #include "memdebug.h"
 
-#define TEST_HANG_TIMEOUT 60 * 1000
+static const char t555_uploadthis[] = "this is the blurb we want to upload\n";
+#define T555_DATALEN (sizeof(t555_uploadthis)-1)
 
-static const char uploadthis[] =
-  "this is the blurb we want to upload\n";
-
-static size_t readcallback(char  *ptr,
-                           size_t size,
-                           size_t nmemb,
-                           void *clientp)
+static size_t t555_read_cb(char *ptr, size_t size, size_t nmemb, void *clientp)
 {
   int *counter = (int *)clientp;
 
   if(*counter) {
     /* only do this once and then require a clearing of this */
-    fprintf(stderr, "READ ALREADY DONE!\n");
+    curl_mfprintf(stderr, "READ ALREADY DONE!\n");
     return 0;
   }
   (*counter)++; /* bump */
 
-  if(size * nmemb >= strlen(uploadthis)) {
-    fprintf(stderr, "READ!\n");
-    strcpy(ptr, uploadthis);
-    return strlen(uploadthis);
+  if(size * nmemb >= T555_DATALEN) {
+    curl_mfprintf(stderr, "READ!\n");
+    strcpy(ptr, t555_uploadthis);
+    return T555_DATALEN;
   }
-  fprintf(stderr, "READ NOT FINE!\n");
+  curl_mfprintf(stderr, "READ NOT FINE!\n");
   return 0;
 }
-static curlioerr ioctlcallback(CURL *handle,
-                               int cmd,
-                               void *clientp)
+
+static curlioerr t555_ioctl_callback(CURL *handle, int cmd, void *clientp)
 {
   int *counter = (int *)clientp;
-  (void)handle; /* unused */
+  (void)handle;
   if(cmd == CURLIOCMD_RESTARTREAD) {
-    fprintf(stderr, "REWIND!\n");
+    curl_mfprintf(stderr, "REWIND!\n");
     *counter = 0; /* clear counter to make the read callback restart */
   }
   return CURLIOE_OK;
 }
 
-
-int test(char *URL)
+static CURLcode test_lib555(const char *URL)
 {
-  int res = 0;
+  CURLcode res = CURLE_OK;
   CURL *curl = NULL;
   int counter = 0;
   CURLM *m = NULL;
@@ -95,21 +87,20 @@ int test(char *URL)
   easy_setopt(curl, CURLOPT_HEADER, 1L);
 
   /* read the POST data from a callback */
-  CURL_IGNORE_DEPRECATION(
-    easy_setopt(curl, CURLOPT_IOCTLFUNCTION, ioctlcallback);
-    easy_setopt(curl, CURLOPT_IOCTLDATA, &counter);
-  )
-  easy_setopt(curl, CURLOPT_READFUNCTION, readcallback);
+  easy_setopt(curl, CURLOPT_IOCTLFUNCTION, t555_ioctl_callback);
+  easy_setopt(curl, CURLOPT_IOCTLDATA, &counter);
+
+  easy_setopt(curl, CURLOPT_READFUNCTION, t555_read_cb);
   easy_setopt(curl, CURLOPT_READDATA, &counter);
   /* We CANNOT do the POST fine without setting the size (or choose
      chunked)! */
-  easy_setopt(curl, CURLOPT_POSTFIELDSIZE, (long)strlen(uploadthis));
+  easy_setopt(curl, CURLOPT_POSTFIELDSIZE, (long)T555_DATALEN);
 
   easy_setopt(curl, CURLOPT_POST, 1L);
   easy_setopt(curl, CURLOPT_PROXY, libtest_arg2);
   easy_setopt(curl, CURLOPT_PROXYUSERPWD, libtest_arg3);
   easy_setopt(curl, CURLOPT_PROXYAUTH,
-                   (long) (CURLAUTH_NTLM | CURLAUTH_DIGEST | CURLAUTH_BASIC) );
+              CURLAUTH_BASIC | CURLAUTH_DIGEST | CURLAUTH_NTLM);
 
   multi_init(m);
 
