@@ -21,28 +21,29 @@
  * SPDX-License-Identifier: curl
  *
  ***************************************************************************/
-#include "test.h"
+#include "first.h"
+
 #include "memdebug.h"
 
 #define THREADS 2
 
 /* struct containing data of a thread */
-struct Tdata {
+struct t586_Tdata {
   CURLSH *share;
-  char *url;
+  const char *url;
 };
 
-struct userdata {
+struct t586_userdata {
   const char *text;
   int counter;
 };
 
 /* lock callback */
-static void my_lock(CURL *handle, curl_lock_data data,
-                    curl_lock_access laccess, void *useptr)
+static void t586_test_lock(CURL *handle, curl_lock_data data,
+                           curl_lock_access laccess, void *useptr)
 {
   const char *what;
-  struct userdata *user = (struct userdata *)useptr;
+  struct t586_userdata *user = (struct t586_userdata *)useptr;
 
   (void)handle;
   (void)laccess;
@@ -61,18 +62,18 @@ static void my_lock(CURL *handle, curl_lock_data data,
       what = "ssl_session";
       break;
     default:
-      fprintf(stderr, "lock: no such data: %d\n", (int)data);
+      curl_mfprintf(stderr, "lock: no such data: %d\n", data);
       return;
   }
-  printf("lock:   %-6s [%s]: %d\n", what, user->text, user->counter);
+  curl_mprintf("lock:   %-6s [%s]: %d\n", what, user->text, user->counter);
   user->counter++;
 }
 
 /* unlock callback */
-static void my_unlock(CURL *handle, curl_lock_data data, void *useptr)
+static void t586_test_unlock(CURL *handle, curl_lock_data data, void *useptr)
 {
   const char *what;
-  struct userdata *user = (struct userdata *)useptr;
+  struct t586_userdata *user = (struct t586_userdata *)useptr;
   (void)handle;
   switch(data) {
     case CURL_LOCK_DATA_SHARE:
@@ -88,96 +89,95 @@ static void my_unlock(CURL *handle, curl_lock_data data, void *useptr)
       what = "ssl_session";
       break;
     default:
-      fprintf(stderr, "unlock: no such data: %d\n", (int)data);
+      curl_mfprintf(stderr, "unlock: no such data: %d\n", data);
       return;
   }
-  printf("unlock: %-6s [%s]: %d\n", what, user->text, user->counter);
+  curl_mprintf("unlock: %-6s [%s]: %d\n", what, user->text, user->counter);
   user->counter++;
 }
 
 /* the dummy thread function */
-static void *fire(void *ptr)
+static void *t586_test_fire(void *ptr)
 {
   CURLcode code;
-  struct Tdata *tdata = (struct Tdata*)ptr;
+  struct t586_Tdata *tdata = (struct t586_Tdata*)ptr;
   CURL *curl;
 
   curl = curl_easy_init();
   if(!curl) {
-    fprintf(stderr, "curl_easy_init() failed\n");
+    curl_mfprintf(stderr, "curl_easy_init() failed\n");
     return NULL;
   }
 
   curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0L);
   curl_easy_setopt(curl, CURLOPT_VERBOSE,    1L);
   curl_easy_setopt(curl, CURLOPT_URL,        tdata->url);
-  printf("CURLOPT_SHARE\n");
+  curl_mprintf("CURLOPT_SHARE\n");
   curl_easy_setopt(curl, CURLOPT_SHARE, tdata->share);
 
-  printf("PERFORM\n");
+  curl_mprintf("PERFORM\n");
   code = curl_easy_perform(curl);
   if(code != CURLE_OK) {
     int i = 0;
-    fprintf(stderr, "perform url '%s' repeat %d failed, curlcode %d\n",
-            tdata->url, i, (int)code);
+    curl_mfprintf(stderr, "perform url '%s' repeat %d failed, curlcode %d\n",
+                  tdata->url, i, code);
   }
 
-  printf("CLEANUP\n");
+  curl_mprintf("CLEANUP\n");
   curl_easy_cleanup(curl);
 
   return NULL;
 }
 
 /* test function */
-CURLcode test(char *URL)
+static CURLcode test_lib586(const char *URL)
 {
   CURLcode res = CURLE_OK;
   CURLSHcode scode = CURLSHE_OK;
-  char *url;
-  struct Tdata tdata;
+  struct t586_Tdata tdata;
   CURL *curl;
   CURLSH *share;
   int i;
-  struct userdata user;
+  struct t586_userdata user;
 
   user.text = "Pigs in space";
   user.counter = 0;
 
-  printf("GLOBAL_INIT\n");
+  curl_mprintf("GLOBAL_INIT\n");
   if(curl_global_init(CURL_GLOBAL_ALL) != CURLE_OK) {
-    fprintf(stderr, "curl_global_init() failed\n");
+    curl_mfprintf(stderr, "curl_global_init() failed\n");
     return TEST_ERR_MAJOR_BAD;
   }
 
   /* prepare share */
-  printf("SHARE_INIT\n");
+  curl_mprintf("SHARE_INIT\n");
   share = curl_share_init();
   if(!share) {
-    fprintf(stderr, "curl_share_init() failed\n");
+    curl_mfprintf(stderr, "curl_share_init() failed\n");
     curl_global_cleanup();
     return TEST_ERR_MAJOR_BAD;
   }
 
   if(CURLSHE_OK == scode) {
-    printf("CURLSHOPT_LOCKFUNC\n");
-    scode = curl_share_setopt(share, CURLSHOPT_LOCKFUNC, my_lock);
+    curl_mprintf("CURLSHOPT_LOCKFUNC\n");
+    scode = curl_share_setopt(share, CURLSHOPT_LOCKFUNC, t586_test_lock);
   }
   if(CURLSHE_OK == scode) {
-    printf("CURLSHOPT_UNLOCKFUNC\n");
-    scode = curl_share_setopt(share, CURLSHOPT_UNLOCKFUNC, my_unlock);
+    curl_mprintf("CURLSHOPT_UNLOCKFUNC\n");
+    scode = curl_share_setopt(share, CURLSHOPT_UNLOCKFUNC, t586_test_unlock);
   }
   if(CURLSHE_OK == scode) {
-    printf("CURLSHOPT_USERDATA\n");
+    curl_mprintf("CURLSHOPT_USERDATA\n");
     scode = curl_share_setopt(share, CURLSHOPT_USERDATA, &user);
   }
   if(CURLSHE_OK == scode) {
-    printf("CURL_LOCK_DATA_SSL_SESSION\n");
+    curl_mprintf("CURL_LOCK_DATA_SSL_SESSION\n");
     scode = curl_share_setopt(share, CURLSHOPT_SHARE,
                               CURL_LOCK_DATA_SSL_SESSION);
   }
 
   if(CURLSHE_OK != scode) {
-    fprintf(stderr, "curl_share_setopt() failed\n");
+    curl_mfprintf(stderr, "curl_share_setopt() failed\n");
     curl_share_cleanup(share);
     curl_global_cleanup();
     return TEST_ERR_MAJOR_BAD;
@@ -192,54 +192,53 @@ CURLcode test(char *URL)
     tdata.share = share;
 
     /* simulate thread, direct call of "thread" function */
-    printf("*** run %d\n",i);
-    fire(&tdata);
+    curl_mprintf("*** run %d\n",i);
+    t586_test_fire(&tdata);
   }
 
 
   /* fetch another one */
-  printf("*** run %d\n", i);
+  curl_mprintf("*** run %d\n", i);
   curl = curl_easy_init();
   if(!curl) {
-    fprintf(stderr, "curl_easy_init() failed\n");
+    curl_mfprintf(stderr, "curl_easy_init() failed\n");
     curl_share_cleanup(share);
     curl_global_cleanup();
     return TEST_ERR_MAJOR_BAD;
   }
 
-  url = URL;
-  test_setopt(curl, CURLOPT_URL, url);
-  printf("CURLOPT_SHARE\n");
+  test_setopt(curl, CURLOPT_URL, URL);
+  curl_mprintf("CURLOPT_SHARE\n");
   test_setopt(curl, CURLOPT_SHARE, share);
 
-  printf("PERFORM\n");
+  curl_mprintf("PERFORM\n");
   res = curl_easy_perform(curl);
 
   /* try to free share, expect to fail because share is in use */
-  printf("try SHARE_CLEANUP...\n");
+  curl_mprintf("try SHARE_CLEANUP...\n");
   scode = curl_share_cleanup(share);
   if(scode == CURLSHE_OK) {
-    fprintf(stderr, "curl_share_cleanup succeed but error expected\n");
+    curl_mfprintf(stderr, "curl_share_cleanup succeed but error expected\n");
     share = NULL;
   }
   else {
-    printf("SHARE_CLEANUP failed, correct\n");
+    curl_mprintf("SHARE_CLEANUP failed, correct\n");
   }
 
 test_cleanup:
 
   /* clean up last handle */
-  printf("CLEANUP\n");
+  curl_mprintf("CLEANUP\n");
   curl_easy_cleanup(curl);
 
   /* free share */
-  printf("SHARE_CLEANUP\n");
+  curl_mprintf("SHARE_CLEANUP\n");
   scode = curl_share_cleanup(share);
   if(scode != CURLSHE_OK)
-    fprintf(stderr, "curl_share_cleanup failed, code errno %d\n",
-            (int)scode);
+    curl_mfprintf(stderr, "curl_share_cleanup failed, code errno %d\n",
+                  scode);
 
-  printf("GLOBAL_CLEANUP\n");
+  curl_mprintf("GLOBAL_CLEANUP\n");
   curl_global_cleanup();
 
   return res;
