@@ -21,29 +21,17 @@
  * SPDX-License-Identifier: curl
  *
  ***************************************************************************/
-#include "curlcheck.h"
+#include "unitcheck.h"
 
 #include "tool_getparam.h"
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include "memdebug.h" /* LAST include file */
 
-static CURLcode unit_setup(void)
+static CURLcode test_tool1394(const char *arg)
 {
-  return CURLE_OK;
-}
+  UNITTEST_BEGIN_SIMPLE
 
-static void unit_stop(void)
-{
-
-}
-
-UNITTEST_START
-
-  const char *values[] = {
+  static const char *values[] = {
     /* -E parameter */        /* exp. cert name */  /* exp. passphrase */
     "foo:bar:baz",            "foo",                "bar:baz",
     "foo\\:bar:baz",          "foo:bar",            "baz",
@@ -86,20 +74,20 @@ UNITTEST_START
       if(certname) {
         if(strcmp(p[1], certname)) {
           printf("expected certname '%s' but got '%s' "
-              "for -E param '%s'\n", p[1], certname, p[0]);
+                 "for -E param '%s'\n", p[1], certname, p[0]);
           fail("assertion failure");
         }
       }
       else {
         printf("expected certname '%s' but got NULL "
-            "for -E param '%s'\n", p[1], p[0]);
+               "for -E param '%s'\n", p[1], p[0]);
         fail("assertion failure");
       }
     }
     else {
       if(certname) {
         printf("expected certname NULL but got '%s' "
-            "for -E param '%s'\n", certname, p[0]);
+               "for -E param '%s'\n", certname, p[0]);
         fail("assertion failure");
       }
     }
@@ -107,20 +95,20 @@ UNITTEST_START
       if(passphrase) {
         if(strcmp(p[2], passphrase)) {
           printf("expected passphrase '%s' but got '%s'"
-              "for -E param '%s'\n", p[2], passphrase, p[0]);
+                 "for -E param '%s'\n", p[2], passphrase, p[0]);
           fail("assertion failure");
         }
       }
       else {
         printf("expected passphrase '%s' but got NULL "
-            "for -E param '%s'\n", p[2], p[0]);
+               "for -E param '%s'\n", p[2], p[0]);
         fail("assertion failure");
       }
     }
     else {
       if(passphrase) {
         printf("expected passphrase NULL but got '%s' "
-            "for -E param '%s'\n", passphrase, p[0]);
+               "for -E param '%s'\n", passphrase, p[0]);
         fail("assertion failure");
       }
     }
@@ -130,4 +118,5 @@ UNITTEST_START
       free(passphrase);
   }
 
-UNITTEST_STOP
+  UNITTEST_END_SIMPLE
+}
