@@ -21,36 +21,37 @@
  * SPDX-License-Identifier: curl
  *
  ***************************************************************************/
-#include "curlcheck.h"
+#include "unitcheck.h"
 
-static CURL *hnd;
-
-static CURLcode unit_setup(void)
+static CURLcode t1396_setup(void)
 {
   CURLcode res = CURLE_OK;
-
   global_init(CURL_GLOBAL_ALL);
   return res;
 }
 
-static void unit_stop(void)
+static void t1396_stop(CURL *easy)
 {
-  if(hnd)
-    curl_easy_cleanup(hnd);
+  if(easy)
+    curl_easy_cleanup(easy);
   curl_global_cleanup();
 }
 
-struct test {
-  const char *in;
-  int inlen;
-  const char *out;
-  int outlen;
-};
-
-UNITTEST_START
+static CURLcode test_unit1396(const char *arg)
 {
+  CURL *easy;
+
+  UNITTEST_BEGIN(t1396_setup())
+
+  struct test {
+    const char *in;
+    int inlen;
+    const char *out;
+    int outlen;
+  };
+
   /* unescape, this => that */
-  const struct test list1[]={
+  const struct test list1[] = {
     {"%61", 3, "a", 1},
     {"%61a", 4, "aa", 2},
     {"%61b", 4, "ab", 2},
@@ -66,7 +67,7 @@ UNITTEST_START
     {NULL, 0, NULL, 0} /* end of list marker */
   };
   /* escape, this => that */
-  const struct test list2[]={
+  const struct test list2[] = {
     {"a", 1, "a", 1},
     {"/", 1, "%2F", 3},
     {"a=b", 3, "a%3Db", 5},
@@ -81,11 +82,11 @@ UNITTEST_START
   };
   int i;
 
-  hnd = curl_easy_init();
-  abort_unless(hnd != NULL, "returned NULL!");
+  easy = curl_easy_init();
+  abort_unless(easy != NULL, "returned NULL!");
   for(i = 0; list1[i].in; i++) {
     int outlen;
-    char *out = curl_easy_unescape(hnd,
+    char *out = curl_easy_unescape(easy,
                                    list1[i].in, list1[i].inlen,
                                    &outlen);
 
@@ -101,7 +102,7 @@ UNITTEST_START
 
   for(i = 0; list2[i].in; i++) {
     int outlen;
-    char *out = curl_easy_escape(hnd, list2[i].in, list2[i].inlen);
+    char *out = curl_easy_escape(easy, list2[i].in, list2[i].inlen);
     abort_unless(out != NULL, "returned NULL!");
 
     outlen = (int)strlen(out);
@@ -113,5 +114,6 @@ UNITTEST_START
 
     curl_free(out);
   }
+
+  UNITTEST_END(t1396_stop(easy))
 }
-UNITTEST_STOP
