@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2021, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,34 +18,35 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
-#include "curlcheck.h"
+#include "unitcheck.h"
 
 #include "speedcheck.h"
 #include "urldata.h"
 
-static CURL *easy;
-
-static CURLcode unit_setup(void)
+static CURLcode t1606_setup(struct Curl_easy **easy)
 {
-  int res = CURLE_OK;
+  CURLcode res = CURLE_OK;
 
   global_init(CURL_GLOBAL_ALL);
-  easy = curl_easy_init();
-  if(!easy) {
+  *easy = curl_easy_init();
+  if(!*easy) {
     curl_global_cleanup();
     return CURLE_OUT_OF_MEMORY;
   }
   return res;
 }
 
-static void unit_stop(void)
+static void t1606_stop(struct Curl_easy *easy)
 {
   curl_easy_cleanup(easy);
   curl_global_cleanup();
 }
 
-static int runawhile(long time_limit,
+static int runawhile(struct Curl_easy *easy,
+                     long time_limit,
                      long speed_limit,
                      curl_off_t speed,
                      int dec)
@@ -75,17 +76,24 @@ static int runawhile(long time_limit,
   return finaltime;
 }
 
-UNITTEST_START
-  fail_unless(runawhile(41, 41, 40, 0) == 41,
+static CURLcode test_unit1606(const char *arg)
+{
+  struct Curl_easy *easy;
+
+  UNITTEST_BEGIN(t1606_setup(&easy))
+
+  fail_unless(runawhile(easy, 41, 41, 40, 0) == 41,
               "wrong low speed timeout");
-  fail_unless(runawhile(21, 21, 20, 0) == 21,
+  fail_unless(runawhile(easy, 21, 21, 20, 0) == 21,
               "wrong low speed timeout");
-  fail_unless(runawhile(60, 60, 40, 0) == 60,
+  fail_unless(runawhile(easy, 60, 60, 40, 0) == 60,
               "wrong log speed timeout");
-  fail_unless(runawhile(50, 50, 40, 0) == 50,
+  fail_unless(runawhile(easy, 50, 50, 40, 0) == 50,
               "wrong log speed timeout");
-  fail_unless(runawhile(40, 40, 40, 0) == 99,
+  fail_unless(runawhile(easy, 40, 40, 40, 0) == 99,
               "should not time out");
-  fail_unless(runawhile(10, 50, 100, 2) == 36,
+  fail_unless(runawhile(easy, 10, 50, 100, 2) == 36,
               "bad timeout");
-UNITTEST_STOP
+
+  UNITTEST_END(t1606_stop(easy))
+}
