@@ -26,7 +26,7 @@
 
 #include "curl_sha512_256.h"
 
-static CURLcode test_unit1615(char *arg)
+static CURLcode test_unit1615(const char *arg)
 {
   UNITTEST_BEGIN_SIMPLE
 
