@@ -5,7 +5,7 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 2020 - 2021, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
@@ -18,40 +18,46 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
+ * SPDX-License-Identifier: curl
+ *
  ***************************************************************************/
-#include "curlcheck.h"
+#include "unitcheck.h"
 
 #include "urldata.h"
 #include "hsts.h"
 
-static CURLcode
-unit_setup(void)
-{
-  return CURLE_OK;
-}
-
-static void
-unit_stop(void)
-{
-  curl_global_cleanup();
-}
-
 #if defined(CURL_DISABLE_HTTP) || defined(CURL_DISABLE_HSTS)
-UNITTEST_START
+static CURLcode test_unit1660(const char *arg)
 {
-  return 0; /* nothing to do when HTTP or HSTS are disabled */
+  UNITTEST_BEGIN_SIMPLE
+  puts("nothing to do when HTTP or HSTS are disabled");
+  UNITTEST_END_SIMPLE
 }
-UNITTEST_STOP
 #else
 
-struct testit {
-  const char *host;
-  const char *chost; /* if non-NULL, use to lookup with */
-  const char *hdr; /* if NULL, just do the lookup */
-  const CURLcode result; /* parse result */
-};
+static void showsts(struct stsentry *e, const char *chost)
+{
+  if(!e)
+    printf("'%s' is not HSTS\n", chost);
+  else {
+    curl_mprintf("%s [%s]: %" CURL_FORMAT_CURL_OFF_T "%s\n",
+                 chost, e->host, e->expires,
+                 e->includeSubDomains ? " includeSubDomains" : "");
+  }
+}
 
-static const struct testit headers[] = {
+static CURLcode test_unit1660(const char *arg)
+{
+  UNITTEST_BEGIN_SIMPLE
+
+  struct testit {
+    const char *host;
+    const char *chost; /* if non-NULL, use to lookup with */
+    const char *hdr; /* if NULL, just do the lookup */
+    const CURLcode result; /* parse result */
+  };
+
+  static const struct testit headers[] = {
   /* two entries read from disk cache, verify first */
   { "-", "readfrom.example", NULL, CURLE_OK},
   { "-", "old.example", NULL, CURLE_OK},
@@ -80,7 +86,7 @@ static const struct testit headers[] = {
   { "2.example.com", NULL,
     "max-age=\"21536000\"; includeSubDomains; includeSubDomains;",
     CURLE_BAD_FUNCTION_ARGUMENT },
-  /* use a unknown directive "include" that should be ignored */
+  /* use an unknown directive "include" that should be ignored */
   { "3.example.com", NULL, "max-age=\"21536000\"; include; includeSubDomains;",
     CURLE_OK },
   /* remove the "3.example.com" one, should still match the example.com */
@@ -101,48 +107,36 @@ static const struct testit headers[] = {
 
   /* make this live for 7 seconds */
   { "expire.example", NULL, "max-age=\"7\"\r\n", CURLE_OK },
-  { NULL, NULL, NULL, 0 }
-};
+  { NULL, NULL, NULL, CURLE_OK }
+  };
 
-static void showsts(struct stsentry *e, const char *chost)
-{
-  if(!e)
-    printf("'%s' is not HSTS\n", chost);
-  else {
-    printf("%s [%s]: %" CURL_FORMAT_CURL_OFF_T "%s\n",
-           chost, e->host, e->expires,
-           e->includeSubDomains ? " includeSubDomains" : "");
-  }
-}
-
-UNITTEST_START
-{
   CURLcode result;
   struct stsentry *e;
   struct hsts *h = Curl_hsts_init();
   int i;
   const char *chost;
   CURL *easy;
-  if(!h)
-    return 1;
+  char savename[256];
+
+  abort_unless(h, "Curl_hsts_init()");
 
   curl_global_init(CURL_GLOBAL_ALL);
   easy = curl_easy_init();
   if(!easy) {
     Curl_hsts_cleanup(&h);
     curl_global_cleanup();
-    return 1;
+    abort_unless(easy, "curl_easy_init()");
   }
 
-  Curl_hsts_loadfile(easy, h, "log/input1660");
+  Curl_hsts_loadfile(easy, h, arg);
 
   for(i = 0; headers[i].host ; i++) {
     if(headers[i].hdr) {
       result = Curl_hsts_parse(h, headers[i].host, headers[i].hdr);
 
       if(result != headers[i].result) {
-        fprintf(stderr, "Curl_hsts_parse(%s) failed: %d\n",
-                headers[i].hdr, result);
+        curl_mfprintf(stderr, "Curl_hsts_parse(%s) failed: %d\n",
+                      headers[i].hdr, result);
         unitfail++;
         continue;
       }
@@ -153,25 +147,26 @@ UNITTEST_START
     }
 
     chost = headers[i].chost ? headers[i].chost : headers[i].host;
-    e = Curl_hsts(h, chost, TRUE);
+    e = Curl_hsts(h, chost, strlen(chost), TRUE);
     showsts(e, chost);
   }
 
-  printf("Number of entries: %zu\n", h->list.size);
+  curl_mprintf("Number of entries: %zu\n", Curl_llist_count(&h->list));
 
   /* verify that it is exists for 7 seconds */
   chost = "expire.example";
   for(i = 100; i < 110; i++) {
-    e = Curl_hsts(h, chost, TRUE);
+    e = Curl_hsts(h, chost, strlen(chost), TRUE);
     showsts(e, chost);
     deltatime++; /* another second passed */
   }
 
-  (void)Curl_hsts_save(easy, h, "log/hsts1660");
+  curl_msnprintf(savename, sizeof(savename), "%s.save", arg);
+  (void)Curl_hsts_save(easy, h, savename);
   Curl_hsts_cleanup(&h);
   curl_easy_cleanup(easy);
   curl_global_cleanup();
-  return unitfail;
+
+  UNITTEST_END(curl_global_cleanup())
 }
-UNITTEST_STOP
 #endif
