@@ -39,7 +39,7 @@ static CURLcode t1664_setup(void)
   return res;
 }
 
-static CURLcode test_unit1664(char *arg)
+static CURLcode test_unit1664(const char *arg)
 {
   UNITTEST_BEGIN(t1664_setup())
 
