@@ -21,36 +21,35 @@
  * SPDX-License-Identifier: curl
  *
  ***************************************************************************/
-#include "curlcheck.h"
-#include "curl_path.h"
+#include "unitcheck.h"
+#include "vssh/curl_path.h"
+#include "memdebug.h"
 
-static CURLcode unit_setup(void)
+static CURLcode test_unit2604(const char *arg)
 {
-  return CURLE_OK;
-}
+  UNITTEST_BEGIN_SIMPLE
 
-static void unit_stop(void)
-{
-}
-
-
-struct set {
-  const char *cp;
-  const char *expect; /* the returned content */
-  const char *next;   /* what cp points to after the call */
-  const char *home;
-  CURLcode result;
-};
-
-UNITTEST_START
 #ifdef USE_SSH
-{
+
+  struct set {
+    const char *cp;
+    const char *expect; /* the returned content */
+    const char *next;   /* what cp points to after the call */
+    const char *home;
+    CURLcode result;
+  };
+
+#if defined(CURL_GNUC_DIAG) || defined(__clang__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Woverlength-strings"
+#endif
+
 /* 60 a's */
 #define SA60 "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa"
 /* 540 a's */
 #define SA540 SA60 SA60 SA60 SA60 SA60 SA60 SA60 SA60 SA60
   int i;
-  size_t too_long = 90720;
+  const size_t too_long = 90720;
   struct set list[] = {
     { "-too-long-", "", "", "", CURLE_TOO_LARGE},
     { SA540 " c", SA540, "c", "/", CURLE_OK},
@@ -72,13 +71,17 @@ UNITTEST_START
     { NULL, NULL, NULL, NULL, CURLE_OK }
   };
 
-  list[0].cp = calloc(1, too_long + 1);
-  fail_unless(list[0].cp, "could not alloc too long value");
-  memset((void *)list[0].cp, 'a', too_long);
+#if defined(CURL_GNUC_DIAG) || defined(__clang__)
+#pragma GCC diagnostic pop
+#endif
+
+  char *cp0 = calloc(1, too_long + 1);
+  fail_unless(cp0, "could not alloc too long value");
+  memset(cp0, 'a', too_long);
 
   for(i = 0; list[i].home; i++) {
     char *path;
-    const char *cp = list[i].cp;
+    const char *cp = i == 0 ? cp0 : list[i].cp;
     CURLcode result = Curl_get_pathname(&cp, &path, list[i].home);
     printf("%u - Curl_get_pathname(\"%s\", ... \"%s\") == %u\n", i,
            list[i].cp, list[i].home, list[i].result);
@@ -98,12 +101,12 @@ UNITTEST_START
         unitfail++;
       }
       curl_free(path);
-
     }
   }
 
-  free((void *)list[0].cp);
-}
+  free(cp0);
+
 #endif
 
-UNITTEST_STOP
+  UNITTEST_END_SIMPLE
+}
