@@ -21,76 +21,68 @@
  * SPDX-License-Identifier: curl
  *
  ***************************************************************************/
-#include "curlcheck.h"
+#include "unitcheck.h"
 #include "curl_get_line.h"
+#include "memdebug.h"
+
+static CURLcode test_unit3200(const char *arg)
+{
+  UNITTEST_BEGIN_SIMPLE
 
 #if !defined(CURL_DISABLE_COOKIES) || !defined(CURL_DISABLE_ALTSVC) ||  \
   !defined(CURL_DISABLE_HSTS) || !defined(CURL_DISABLE_NETRC)
 
-/* The test XML does not supply a way to write files without newlines
- * so we write our own
- */
+#if defined(CURL_GNUC_DIAG) || defined(__clang__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Woverlength-strings"
+#endif
+
+  /* The test XML does not supply a way to write files without newlines
+   * so we write our own
+   */
 
 #define C64 "0123456789abcdef0123456789abcdef0123456789abcdef0123456789abcdef"
 #define C256 C64 C64 C64 C64
 #define C1024 C256 C256 C256 C256
 #define C4096 C1024 C1024 C1024 C1024
 
-static CURLcode unit_setup(void)
-{
-  return CURLE_OK;
-}
+  static const char *filecontents[] = {
+    /* Both should be read */
+    "LINE1\n"
+    "LINE2 NEWLINE\n",
 
-static CURLcode unit_stop(void)
-{
-  return CURLE_OK;
-}
+    /* Both should be read */
+    "LINE1\n"
+    "LINE2 NONEWLINE",
 
-#ifdef __GNUC__
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Woverlength-strings"
+    /* Only first should be read */
+    "LINE1\n"
+    C4096,
+
+    /* First line should be read */
+    "LINE1\n"
+    C4096 "SOME EXTRA TEXT",
+
+    /* Only first should be read */
+    "LINE1\n"
+    C4096 "SOME EXTRA TEXT\n"
+    "LINE3\n",
+
+    "LINE1\x1aTEST"
+  };
+
+#if defined(CURL_GNUC_DIAG) || defined(__clang__)
+#pragma GCC diagnostic pop
 #endif
 
-#define NUMTESTS 6
-static const char *filecontents[] = {
-  /* Both should be read */
-  "LINE1\n"
-  "LINE2 NEWLINE\n",
-
-  /* Both should be read */
-  "LINE1\n"
-  "LINE2 NONEWLINE",
-
-  /* Only first should be read */
-  "LINE1\n"
-  C4096,
-
-  /* First line should be read */
-  "LINE1\n"
-  C4096 "SOME EXTRA TEXT",
-
-  /* Only first should be read */
-  "LINE1\n"
-  C4096 "SOME EXTRA TEXT\n"
-  "LINE3\n",
-
-  "LINE1\x1aTEST"
-};
-
-#ifdef __GNUC__
-#pragma GCC diagnostic warning "-Woverlength-strings"
-#endif
-
-
-UNITTEST_START
   size_t i;
   int rc = 0;
-  for(i = 0; i < NUMTESTS; i++) {
+  for(i = 0; i < CURL_ARRAYSIZE(filecontents); i++) {
     FILE *fp;
     struct dynbuf buf;
-    int len = 4096;
+    size_t len = 4096;
     char *line;
-    Curl_dyn_init(&buf, len);
+    curlx_dyn_init(&buf, len);
 
     fp = fopen(arg, "wb");
     abort_unless(fp != NULL, "Cannot open testfile");
@@ -100,91 +92,78 @@ UNITTEST_START
     fp = fopen(arg, "rb");
     abort_unless(fp != NULL, "Cannot open testfile");
 
-    fprintf(stderr, "Test %zd...", i);
+    curl_mfprintf(stderr, "Test %zd...", i);
     switch(i) {
       case 0:
         rc = Curl_get_line(&buf, fp);
-        line = Curl_dyn_ptr(&buf);
-        fail_unless(line && !strcmp("LINE1\n", line),
+        line = curlx_dyn_ptr(&buf);
+        fail_unless(rc && line && !strcmp("LINE1\n", line),
                     "First line failed (1)");
         rc = Curl_get_line(&buf, fp);
-        line = Curl_dyn_ptr(&buf);
-        fail_unless(line && !strcmp("LINE2 NEWLINE\n", line),
+        line = curlx_dyn_ptr(&buf);
+        fail_unless(rc && line && !strcmp("LINE2 NEWLINE\n", line),
                     "Second line failed (1)");
         rc = Curl_get_line(&buf, fp);
-        abort_unless(!Curl_dyn_len(&buf), "Missed EOF (1)");
+        abort_unless(!curlx_dyn_len(&buf), "Missed EOF (1)");
         break;
       case 1:
         rc = Curl_get_line(&buf, fp);
-        line = Curl_dyn_ptr(&buf);
-        fail_unless(line && !strcmp("LINE1\n", line),
+        line = curlx_dyn_ptr(&buf);
+        fail_unless(rc && line && !strcmp("LINE1\n", line),
                     "First line failed (2)");
         rc = Curl_get_line(&buf, fp);
-        line = Curl_dyn_ptr(&buf);
-        fail_unless(line && !strcmp("LINE2 NONEWLINE\n", line),
+        line = curlx_dyn_ptr(&buf);
+        fail_unless(rc && line && !strcmp("LINE2 NONEWLINE\n", line),
                     "Second line failed (2)");
         rc = Curl_get_line(&buf, fp);
-        abort_unless(!Curl_dyn_len(&buf), "Missed EOF (2)");
+        abort_unless(!curlx_dyn_len(&buf), "Missed EOF (2)");
         break;
       case 2:
         rc = Curl_get_line(&buf, fp);
-        line = Curl_dyn_ptr(&buf);
-        fail_unless(line && !strcmp("LINE1\n", line),
+        line = curlx_dyn_ptr(&buf);
+        fail_unless(rc && line && !strcmp("LINE1\n", line),
                     "First line failed (3)");
         rc = Curl_get_line(&buf, fp);
-        fail_unless(!Curl_dyn_len(&buf),
+        fail_unless(!curlx_dyn_len(&buf),
                     "Did not detect max read on EOF (3)");
         break;
       case 3:
         rc = Curl_get_line(&buf, fp);
-        line = Curl_dyn_ptr(&buf);
-        fail_unless(line && !strcmp("LINE1\n", line),
+        line = curlx_dyn_ptr(&buf);
+        fail_unless(rc && line && !strcmp("LINE1\n", line),
                     "First line failed (4)");
         rc = Curl_get_line(&buf, fp);
-        fail_unless(!Curl_dyn_len(&buf),
+        fail_unless(!curlx_dyn_len(&buf),
                     "Did not ignore partial on EOF (4)");
         break;
       case 4:
         rc = Curl_get_line(&buf, fp);
-        line = Curl_dyn_ptr(&buf);
-        fail_unless(line && !strcmp("LINE1\n", line),
+        line = curlx_dyn_ptr(&buf);
+        fail_unless(rc && line && !strcmp("LINE1\n", line),
                     "First line failed (5)");
         rc = Curl_get_line(&buf, fp);
-        fail_unless(!Curl_dyn_len(&buf),
+        fail_unless(!curlx_dyn_len(&buf),
                     "Did not bail out on too long line");
         break;
       case 5:
         rc = Curl_get_line(&buf, fp);
-        line = Curl_dyn_ptr(&buf);
-        fail_unless(line && !strcmp("LINE1\x1aTEST\n", line),
+        line = curlx_dyn_ptr(&buf);
+        fail_unless(rc && line && !strcmp("LINE1\x1aTEST\n", line),
                     "Missed/Misinterpreted ^Z (6)");
         rc = Curl_get_line(&buf, fp);
-        abort_unless(!Curl_dyn_len(&buf), "Missed EOF (6)");
+        abort_unless(!curlx_dyn_len(&buf), "Missed EOF (6)");
         break;
       default:
         abort_unless(1, "Unknown case");
         break;
     }
-    Curl_dyn_free(&buf);
+    curlx_dyn_free(&buf);
     fclose(fp);
-    fprintf(stderr, "OK\n");
+    curl_mfprintf(stderr, "OK\n");
   }
-  return rc;
-UNITTEST_STOP
-
-#ifdef __GNUC__
-#pragma GCC diagnostic pop
-#endif
-
-#else
-static CURLcode unit_setup(void)
-{
-  return CURLE_OK;
-}
-static void unit_stop(void)
-{
-}
-UNITTEST_START
-UNITTEST_STOP
+  return (CURLcode)rc;
 
 #endif
+
+  UNITTEST_END_SIMPLE
+}
